#include "../common/common.h"

#include "stack.h"
#include "csg.h"

/* disk command */
class disk : public textcommand
{
public:
	IMPLEMENT_STORAGE;

	disk(void)
	{
		this->m_cx = 0;
		this->m_cy = 0;
		this->m_radius = 0;
	}

	disk(const std::vector<std::string>& args)
	{
		if (args.size() != 3)
			throw_exception(invalid_number_of_arguments_exception);

		this->m_cx = atof(args[0].c_str());
		this->m_cy = atof(args[1].c_str());
		this->m_radius = atof(args[2].c_str());
	}

	virtual void run(void) const override
	{
		stack_push(std::make_shared<csg::disk>(this->m_cx, this->m_cy, this->m_radius));
	}

	virtual std::string log(void) const override
	{
		return get_instance<library::manager>()->get_name() + "/" + format("disk %f %f %f", this->m_cx, this->m_cy, this->m_radius);
	}

private:
	double m_cx, m_cy, m_radius;
};

BEGIN_STORAGE(disk)
{
	DECLARE_STORAGE(disk, m_cx),
	DECLARE_STORAGE(disk, m_cy),
	DECLARE_STORAGE(disk, m_radius),
} END_STORAGE(disk, version_command_t);

DECLARE_INSTANCEABLE_OBJECT(disk, version_command_t);

DECLARE_COMMAND_EX(disk, "disk");