#include "../common/common.h"

#include "stack.h"
#include "csg.h"

/* rect command */
class rect : public textcommand
{
public:
	IMPLEMENT_STORAGE;

	rect(void)
	{
		this->m_cx = 0;
		this->m_cy = 0;
		this->m_width = 0;
		this->m_height = 0;
	}

	rect(const std::vector<std::string>& args)
	{
		if (args.size() != 4)
			throw_exception(invalid_number_of_arguments_exception);

		this->m_cx = atof(args[0].c_str());
		this->m_cy = atof(args[1].c_str());
		this->m_width = atof(args[2].c_str());
		this->m_height = atof(args[3].c_str());
	}

	virtual void run(void) const override
	{
		stack_push(std::make_shared<csg::rect>(this->m_cx, this->m_cy, this->m_width, this->m_height));
	}

	virtual std::string log(void) const override
	{
		return get_instance<library::manager>()->get_name() + "/" + format("rect %f %f %f %f", this->m_cx, this->m_cy, this->m_width, this->m_height);
	}

private:
	double m_cx, m_cy, m_width, m_height;
};

BEGIN_STORAGE(rect)
{
	DECLARE_STORAGE(rect, m_cx),
	DECLARE_STORAGE(rect, m_cy),
	DECLARE_STORAGE(rect, m_width),
	DECLARE_STORAGE(rect, m_height),
} END_STORAGE(rect, version_command_t);

DECLARE_INSTANCEABLE_OBJECT(rect, version_command_t);

DECLARE_COMMAND_EX(rect, "rect");