#include "../common/common.h"

#include "stack.h"
#include "csg.h"

/* unknown_param_exception exception class */
class unknown_param_exception : public exception::base
{
public:
	unknown_param_exception(const std::string& name)
	{
		this->m_name = name;
	}

	virtual std::string to_string(void) const override
	{
		return "Unknown parameter \"" + this->m_name + "\" !";
	}

private:
	std::string m_name;
};

/* set command */
class set : public textcommand
{
public:
	IMPLEMENT_STORAGE;

	set(void) {}

	set(const std::vector<std::string>& args)
	{
		if (args.size() != 2)
			throw_exception(invalid_number_of_arguments_exception);

		this->m_name = args[0];
		this->m_value = args[1];
	}

	virtual void run(void) const override
	{
		auto p = stack_top();

		if (!p->set_param(this->m_name, this->m_value))
			throw_exception(unknown_param_exception, this->m_name);
	}

	virtual std::string log(void) const override
	{
		return get_instance<library::manager>()->get_name() + "/" + format("set \"%s\" \"%s\"", this->m_name.c_str(), this->m_value.c_str());
	}

private:
	storage::ext::string m_name, m_value;
};

BEGIN_STORAGE(set)
{
	DECLARE_STORAGE(set, m_name),
	DECLARE_STORAGE(set, m_value),
} END_STORAGE(set, version_command_t);

DECLARE_INSTANCEABLE_OBJECT(set, version_command_t);

DECLARE_COMMAND_EX(set, "set");