#pragma once

#include "../core/utils/utils.h"
#include "../core/resources/uid.h"
#include "../core/strings/format.h"

#include <list>
#include <string>
#include <memory>

namespace graphml
{
	class object
	{
	public:
		virtual std::list<std::string> to_string(void) const = 0;
	};

	class shape : public object
	{
	public:
	};

	class edge : public object
	{
	public:
	};

	class file
	{
	public:
		file(void) {}

		std::list<std::string> to_string(void) const
		{
			std::list<std::string> ret;

			ret.push_back("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			ret.push_back("<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:y=\"http://www.yworks.com/xml/graphml\" xmlns:yed=\"http://www.yworks.com/xml/yed/3\" xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd\">");

			for (auto& v : this->m_shapes)
				ret.push_back(format("  <key for=\"node\" id=\"data-%s\" yfiles.type=\"nodegraphics\" />", v.first.c_str()));

			for (auto& v : this->m_edges)
			{
				std::string id, from, to;
				std::shared_ptr<const edge> p;

				std::tie(id, from, to, p) = v;

				ret.push_back(format("  <key for=\"edge\" id=\"data-%s\" yfiles.type=\"edgegraphics\" />", id.c_str()));
			}

			ret.push_back("");

			ret.push_back("  <graph id=\"flowchart\" edgedefault=\"directed\">");
			ret.push_back("");

			for (auto& v : this->m_shapes)
			{
				ret.push_back(format("    <node id=\"%s\">", v.first.c_str()));
				ret.push_back(format("      <data key=\"data-%s\">", v.first.c_str()));
				ret.push_back("        <y:ShapeNode>");

				if (v.second != nullptr)
					ret.splice(ret.end(), v.second->to_string());

				ret.push_back("        </y:ShapeNode>");
				ret.push_back("      </data>");
				ret.push_back("    </node>");
				ret.push_back("");
			}

			for (auto& v : this->m_edges)
			{
				std::string id, from, to;
				std::shared_ptr<const edge> p;

				std::tie(id, from, to, p) = v;

				ret.push_back(format("    <edge id=\"%s\" source=\"%s\" target=\"%s\">", id.c_str(), from.c_str(), to.c_str()));

				ret.push_back(format("      <data key=\"data-%s\">", id.c_str()));
				ret.push_back("        <y:PolyLineEdge>");

				if (p != nullptr)
					ret.splice(ret.end(), p->to_string());

				ret.push_back("        </y:PolyLineEdge>");
				ret.push_back("      </data>");

				ret.push_back("    </edge>");
				ret.push_back("");
			}

			ret.push_back("  </graph>");
			ret.push_back("</graphml>");

			return ret;
		}

		auto add(std::shared_ptr<const shape> p)
		{
			auto id = get_instance<resources::uid::manager>()->uid();

			this->m_shapes.emplace_back(std::make_pair(id, p));

			return id;
		}

		auto add(const std::string& from, const std::string& to, std::shared_ptr<const edge> p)
		{
			auto id = get_instance<resources::uid::manager>()->uid();

			this->m_edges.emplace_back(std::tuple<std::string, std::string, std::string, std::shared_ptr<const edge>>(id, from, to, p));

			return id;
		}

	private:
		std::list<std::pair<std::string, std::shared_ptr<const shape>>> m_shapes;
		std::list<std::tuple<std::string, std::string, std::string, std::shared_ptr<const edge>>> m_edges;
	};
};